# Generated by roxygen2: do not edit by hand

S3method(print,cat2cat_ml_run)
export(cat2cat)
export(cat2cat_agg)
export(cat2cat_ml_run)
export(cat_apply_freq)
export(cross_c2c)
export(dummy_c2c)
export(get_freqs)
export(get_mappings)
export(plot_c2c)
export(prune_c2c)
export(summary_c2c)
importFrom(MASS,lda)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(graphics,par)
importFrom(stats,complete.cases)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,setNames)
importFrom(utils,head)
