% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cat2cat.R
\name{validate_data}
\alias{validate_data}
\title{Validate cat2cat data}
\usage{
validate_data(data)
}
\description{
Validate cat2cat data
}
\keyword{internal}
